list(
  # Adjustment name ----
  Name = "Reporting Delays",

  # Input parameters to the adjustment function ----
  Parameters = list(
    # Parameter 1: a specification with label, type and default value
    startYear = list(
      label = "Start year",
      value = 2000L,
      input = "numeric")
  ),

  # Names of packages that must be made available to the adjustment function ----
  RequiredPackageNames = c(),

  ## Adjustment function ----
  AdjustmentFunction = function(inputData, parameters) {

    startYear <- parameters$startYear

    outputData <- copy(inputData)

    outputData[, ":="(
      NotificationTime = DateOfNotificationYear + 1/4 * DateOfNotificationQuarter,
      DiagnosisTime = DateOfDiagnosisYear + 1/4 * DateOfDiagnosisQuarter
    )]
    outputData[, VarX := 4 * (NotificationTime - DiagnosisTime)]
    outputData[VarX < 0, VarX := NA]

    outputData[, ":="(
      MinNotificationTime = min(NotificationTime),
      MaxNotificationTime = max(NotificationTime)
    ), by = .(ReportingCountry)]
    outputData[, ":="(
      VarT = 4 * (MaxNotificationTime - DiagnosisTime),
      ReportingDelay = 1L,
      Tf = 4 * (MaxNotificationTime - max(startYear, MinNotificationTime))
    )]
    outputData[, ":="(
      VarXs = Tf - VarX,
      VarTs = Tf - VarT
    )]

    model <- outputData[, survival::Surv(time = VarTs,
                                         time2 = VarTs + VarXs,
                                         event = ReportingDelay)]
    fit <- survival::survfit(model ~ 1)

    final <- data.table(Delay = fit$time,
                        P = fit$surv,
                        StErr = fit$std.err,
                        Var = fit$std.err^2)
    final[, Delay := max(Delay) - Delay - 1]
    final <- final[Delay >= 0]

    agregat <- outputData[, .(Count = .N), by = .(DiagnosisTime, ReportingCountry, VarT)]
    agregat <- merge(agregat, final,
                     by.x = c("VarT"),
                     by.y = c("Delay"))
    agregat[, ":="(
      EstCount = Count / P,
      EstCountVar = (Count * (Count + 1) / P^4 * Var) + Count * (1 - P) / P^2
    )]
    agregat[, ":="(
      LowerEstCount = EstCount - 1.96 * sqrt(EstCountVar),
      UpperEstCount = EstCount + 1.96 * sqrt(EstCountVar)
    )]

    outPlot <- ggplot2::ggplot(agregat, aes(x = DiagnosisTime)) +
      ggplot2::geom_ribbon(aes(ymin = pmin(pmax(LowerEstCount, 0), max(EstCount)),
                               ymax = pmin(pmax(UpperEstCount, 0), max(EstCount)),
                               fill = "Uncertainty interval\nfor estimated total count"),
                           alpha = 0.4) +
      ggplot2::geom_point(aes(y = Count, color = "Reported count"), size = 1) +
      ggplot2::geom_point(aes(y = EstCount, color = "Estimated total count"), size = 1) +
      scale_fill_manual("", values = "grey70") +
      scale_colour_manual("", values = c("red", "blue")) +
      ggtitle("Reported and estimated total count of cases") +
      xlab("Reporting date") +
      ylab("Count of HIV cases")

    artifacts <- list(OutputPlot = outPlot)

    return(list(Table = agregat,
                Artifacts = artifacts))
  }
)
